# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.
import re

from io import StringIO

from django.core.management import call_command
from django.db.utils import DEFAULT_DB_ALIAS
from django.http import HttpResponse

from smuggler import settings


def save_uploaded_file_on_disk(uploaded_file, destination_path):
    with open(destination_path, 'wb') as fp:
        for chunk in uploaded_file.chunks():
            fp.write(chunk)


def serialize_to_response(app_labels=None, exclude=None, response=None,
                          format=settings.SMUGGLER_FORMAT,
                          indent=settings.SMUGGLER_INDENT):
    app_labels = app_labels or []
    exclude = exclude or []
    response = response or HttpResponse(content_type='text/plain')
    stream = StringIO()
    error_stream = StringIO()
    call_command('dumpdata', *app_labels, **{
        'stdout': stream,
        'stderr': error_stream,
        'exclude': exclude,
        'format': format,
        'indent': indent,
        'use_natural_foreign_keys': True,
        'use_natural_primary_keys': True
    })
    response.write(stream.getvalue())
    return response


def load_fixtures(fixtures):
    stream = StringIO()
    error_stream = StringIO()
    call_command('loaddata', *fixtures, **{
        'stdout': stream,
        'stderr': error_stream,
        'ignore': True,
        'database': DEFAULT_DB_ALIAS,
        'verbosity': 1
    })
    stream.seek(0)
    result = stream.read()
    return int(re.match(r'Installed\s([0-9]+)\s.*', result).groups()[0])



import os
from unittest import TestCase

from django.core.files.uploadedfile import SimpleUploadedFile

from smuggler.utils import save_uploaded_file_on_disk


def p(*args):
    return os.path.abspath(
        os.path.join(os.path.dirname(__file__), *args))


class TestSaveUploadedFileOnDisk(TestCase):
    def test_save_uploaded_file_on_disk(self):
        path = p('..', 'smuggler_fixtures', 'test.json')
        save_uploaded_file_on_disk(
            SimpleUploadedFile('test.json', b'[]'), path)
        self.assertTrue(os.path.exists(path))
        os.unlink(path)



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

"""
Django Smuggler is a pluggable application for Django Web Framework that help
you dump/load fixtures via the automatically-generated admin interface.
"""

VERSION = (1, 0, 2)


def get_version():
    """
    Returns the version as a human-format string.
    """
    return ".".join([str(i) for i in VERSION])


__author__ = "See the file AUTHORS."
__license__ = "GNU Lesser General Public License v3 or later (LGPLv3+)"
__url__ = "https://github.com/semente/django-smuggler"
__version__ = get_version()



from django.contrib.auth.models import Permission, User
from django.test import TestCase
from django.urls import reverse


class TestAdminNormalUser(TestCase):
    def setUp(self):
        staff = User(username='staff')
        staff.set_password('test')
        staff.is_staff = True
        staff.save()
        staff.user_permissions.add(
            Permission.objects.get_by_natural_key(
                'change_page', 'test_app', 'page'))
        self.url = reverse('admin:test_app_page_changelist')
        self.client.login(username='staff', password='test')

    def test_has_no_load_button(self):
        response = self.client.get(self.url)
        self.assertNotContains(response, '<a href="/admin/load/">')

    def test_has_no_dump_button(self):
        response = self.client.get(self.url)
        self.assertNotContains(response, '<a href="dump/">')


class TestAdminSuperUser(TestCase):
    def setUp(self):
        superuser = User(username='superuser')
        superuser.set_password('test')
        superuser.is_staff = True
        superuser.is_superuser = True
        superuser.save()
        self.url = reverse('admin:test_app_page_changelist')
        self.client.login(username='superuser', password='test')

    def test_has_load_button(self):
        response = self.client.get(self.url)
        self.assertContains(response, '<a href="/admin/load/">')

    def test_has_dump_button(self):
        response = self.client.get(self.url)
        self.assertContains(response, '<a href="dump/">')



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

from django.conf import settings
from django.dispatch import receiver
from django.test.signals import setting_changed

SMUGGLER_EXCLUDE_LIST = getattr(settings, 'SMUGGLER_EXCLUDE_LIST', [])
SMUGGLER_FIXTURE_DIR = getattr(settings, 'SMUGGLER_FIXTURE_DIR', None)
SMUGGLER_FORMAT = getattr(settings, 'SMUGGLER_FORMAT', 'json')
SMUGGLER_INDENT = getattr(settings, 'SMUGGLER_INDENT', 2)


@receiver(setting_changed)
def update_settings(setting=None, value=None, **kwargs):
    if setting and setting in globals():  # pragma: no branch
        globals()[setting] = value



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.
import os.path
import tempfile
from datetime import datetime

from django.contrib import messages
from django.contrib.admin.helpers import AdminForm
from django.contrib.auth.decorators import user_passes_test
from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
from django.core.management.base import CommandError
from django.core.serializers.base import DeserializationError
from django.db import IntegrityError
from django.http import HttpResponseRedirect
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from django.views.generic.edit import FormView

from smuggler import settings
from smuggler.forms import ImportForm
from smuggler.utils import (load_fixtures, save_uploaded_file_on_disk,
                            serialize_to_response)


def dump_to_response(request, app_label=None, exclude=None,
                     filename_prefix=None):
    """Utility function that dumps the given app/model to an HttpResponse.
    """
    app_label = app_label or []
    exclude = exclude
    try:
        filename = '%s.%s' % (datetime.now().isoformat(),
                              settings.SMUGGLER_FORMAT)
        if filename_prefix:
            filename = '%s_%s' % (filename_prefix, filename)
        if not isinstance(app_label, list):
            app_label = [app_label]
        response = serialize_to_response(app_label, exclude)
        response['Content-Disposition'] = 'attachment; filename=%s' % filename
        return response
    except CommandError as e:
        messages.error(
            request,
            _('An exception occurred while dumping data: %s') % force_str(e))
    return HttpResponseRedirect(request.build_absolute_uri().split('dump')[0])


def is_superuser(u):
    if u.is_authenticated:
        if u.is_superuser:
            return True
        raise PermissionDenied
    return False


@user_passes_test(is_superuser)
def dump_data(request):
    """Exports data from whole project.
    """
    # Try to grab app_label data
    app_label = request.GET.get('app_label', [])
    if app_label:
        app_label = app_label.split(',')
    return dump_to_response(request, app_label=app_label,
                            exclude=settings.SMUGGLER_EXCLUDE_LIST)


@user_passes_test(is_superuser)
def dump_app_data(request, app_label):
    """Exports data from a application.
    """
    return dump_to_response(request, app_label, settings.SMUGGLER_EXCLUDE_LIST,
                            app_label)


@user_passes_test(is_superuser)
def dump_model_data(request, app_label, model_label):
    """Exports data from a model.
    """
    return dump_to_response(request, '%s.%s' % (app_label, model_label),
                            [], '-'.join((app_label, model_label)))


class LoadDataView(FormView):
    form_class = ImportForm
    template_name = 'smuggler/load_data_form.html'
    success_url = '.'

    def form_valid(self, form):
        uploads = form.cleaned_data.get('uploads', [])
        store = form.cleaned_data.get('store', False)
        picked_files = form.cleaned_data.get('picked_files', [])
        fixtures = []
        tmp_fixtures = []
        for upload in uploads:
            file_name = upload.name
            if store:  # Store the file in SMUGGLER_FIXTURE_DIR
                destination_path = os.path.join(
                    settings.SMUGGLER_FIXTURE_DIR, file_name)
                save_uploaded_file_on_disk(upload, destination_path)
            else:  # Store the file in a tmp file
                prefix, suffix = os.path.splitext(file_name)
                destination_path = tempfile.mkstemp(
                    suffix=suffix, prefix=prefix + '_')[1]
                save_uploaded_file_on_disk(upload, destination_path)
                tmp_fixtures.append(destination_path)
            fixtures.append(destination_path)
        for file_name in picked_files:
            fixtures.append(file_name)
        try:
            obj_count = load_fixtures(fixtures)
            user_msg = ' '.join([
                ngettext_lazy(
                    'Successfully imported %(count)d file.',
                    'Successfully imported %(count)d files.',
                    len(fixtures)
                ) % {'count': len(fixtures)},
                ngettext_lazy(
                    'Loaded %(count)d object.',
                    'Loaded %(count)d objects.',
                    obj_count
                ) % {'count': obj_count}])
            messages.info(self.request, user_msg)
        except (IntegrityError, ObjectDoesNotExist,
                DeserializationError, CommandError) as e:
            messages.error(
                self.request,
                _('An exception occurred while loading data: %s') % str(e))
        finally:
            # Remove our tmp files
            for tmp_file in tmp_fixtures:
                os.unlink(tmp_file)
        return super().form_valid(form)

    def get_admin_form(self, form):
        return AdminForm(form, self.get_fieldsets(form), {})

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['adminform'] = self.get_admin_form(context['form'])
        return context

    def get_fieldsets(self, form):
        fields = form.fields.keys()
        if 'picked_files' in fields:
            return [
                (_('Upload'), {'fields': ['uploads', 'store']}),
                (_('From fixture directory'), {'fields': ['picked_files']})
            ]
        return [(None, {'fields': fields})]


load_data = user_passes_test(is_superuser)(LoadDataView.as_view())



from django.test import TestCase
from django.urls import reverse


class TestSmugglerUrls(TestCase):
    def test_can_reverse_dump_data(self):
        self.assertEqual(reverse('dump-data'), '/admin/dump/')

    def test_can_reverse_dump_app_data(self):
        url = reverse('dump-app-data', kwargs={'app_label': 'sites'})
        self.assertEqual(url, '/admin/sites/dump/')

    def test_can_reverse_dump_model_data(self):
        url = reverse('dump-model-data', kwargs={
            'app_label': 'sites',
            'model_label': 'site'
        })
        self.assertEqual(url, '/admin/sites/site/dump/')

    def test_can_reverse_load_data(self):
        self.assertEqual(reverse('load-data'), '/admin/load/')



from django.conf import settings
from django.conf.urls.static import static
from django.contrib import admin
from django.urls import include, path

admin.autodiscover()

urlpatterns = [
    path('admin/', include('smuggler.urls')),
    path('admin/', admin.site.urls)
] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)



import json
import os.path

from django.contrib import messages
from django.contrib.auth.models import User
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase, TransactionTestCase
from django.test.utils import override_settings
from django.urls import reverse
from freezegun import freeze_time

from smuggler.forms import ImportForm
from tests.test_app.models import Page


def p(*args):
    return os.path.abspath(
        os.path.join(os.path.dirname(__file__), *args))


class SuperUserTestCase:
    def setUp(self):
        super().setUp()
        superuser = User(username='superuser')
        superuser.set_password('test')
        superuser.is_staff = True
        superuser.is_superuser = True
        superuser.save()
        self.client.login(username='superuser', password='test')


class TestDumpViewsGenerateDownloadsWithSaneFilenames(SuperUserTestCase,
                                                      TestCase):
    @freeze_time('2012-01-14')
    def test_dump_data(self):
        url = reverse('dump-data')
        response = self.client.get(url)
        self.assertEqual(response['Content-Disposition'],
                         'attachment; filename=2012-01-14T00:00:00.json')

    @freeze_time('2012-01-14')
    def test_dump_app_data(self):
        url = reverse('dump-app-data', kwargs={'app_label': 'sites'})
        response = self.client.get(url)
        self.assertEqual(response['Content-Disposition'],
                         'attachment; filename=sites_2012-01-14T00:00:00.json')

    @freeze_time('2012-01-14')
    def test_dump_model_data(self):
        url = reverse('dump-model-data', kwargs={
            'app_label': 'sites',
            'model_label': 'site'
        })
        response = self.client.get(url)
        self.assertEqual(response['Content-Disposition'],
                         'attachment;'
                         ' filename=sites-site_2012-01-14T00:00:00.json')


class TestDumpData(SuperUserTestCase, TestCase):
    def test_dump_data_parameters(self):
        url = reverse('dump-data')
        response = self.client.get(url, {
            'app_label': 'auth.user,sites'
        })
        content = json.loads(response.content.decode('utf-8'))
        self.assertTrue([i for i in content if i['model'] == 'auth.user'])
        self.assertTrue([i for i in content if i['model'] == 'sites.site'])


class TestDumpHandlesErrorsGracefully(SuperUserTestCase, TestCase):
    def test_erroneous_dump_has_error_messages(self):
        url = reverse('dump-app-data', kwargs={'app_label': 'flatpages'})
        response = self.client.get(url, follow=True)
        response_messages = list(response.context['messages'])
        self.assertEqual(1, len(response_messages))
        self.assertEqual(messages.ERROR, response_messages[0].level)
        self.assertRegex(response_messages[0].message,
                    r'An exception occurred while dumping data:.*flatpages.*')

    def test_erroneous_dump_redirects(self):
        url = reverse('dump-app-data', kwargs={'app_label': 'flatpages'})
        response = self.client.get(url)
        self.assertRedirects(
            response, 'http://testserver/admin/flatpages/',
            target_status_code=404)


class TestLoadDataGet(SuperUserTestCase, TestCase):
    def setUp(self):
        super().setUp()
        self.url = reverse('load-data')

    def test_renders_correct_template(self):
        response = self.client.get(self.url)
        self.assertTemplateUsed(response, 'smuggler/load_data_form.html')

    def test_has_form_in_context(self):
        response = self.client.get(self.url)
        self.assertIsInstance(response.context['form'],
                              ImportForm)


class TestLoadDataPost(SuperUserTestCase, TransactionTestCase):
    def setUp(self):
        super().setUp()
        self.url = reverse('load-data')

    def test_load_fixture(self):
        self.assertEqual(0, Page.objects.count())
        with open(
            p('..', 'smuggler_fixtures', 'page_dump.json'), mode='rb'
        ) as f:
            self.client.post(self.url, {
                'uploads': f
            }, follow=True)
        self.assertEqual(1, Page.objects.count())

    def test_load_fixture_message(self):
        with open(
            p('..', 'smuggler_fixtures', 'page_dump.json'), mode='rb'
        ) as f:
            response = self.client.post(self.url, {
                'uploads': f
            }, follow=True)
        response_messages = list(response.context['messages'])
        self.assertEqual(1, len(response_messages))
        self.assertEqual(messages.INFO, response_messages[0].level)
        self.assertEqual(response_messages[0].message,
                         'Successfully imported 1 file. Loaded 1 object.')

    @override_settings(FILE_UPLOAD_MAX_MEMORY_SIZE=0)
    def test_load_fixture_with_chunks(self):
        self.assertEqual(0, Page.objects.count())
        with open(
            p('..', 'smuggler_fixtures', 'big_file.json'), mode='rb'
        ) as f:
            self.client.post(self.url, {
                'uploads': f
            }, follow=True)
        self.assertEqual(1, Page.objects.count())

    def test_handle_garbage_upload(self):
        with open(
            p('..', 'smuggler_fixtures', 'garbage', 'garbage.json'), mode='rb'
        ) as f:
            response = self.client.post(self.url, {
                'uploads': f
            }, follow=True)
        response_messages = list(response.context['messages'])
        self.assertEqual(1, len(response_messages))
        self.assertEqual(messages.ERROR, response_messages[0].level)
        self.assertRegex(response_messages[0].message,
                    'An exception occurred while loading data: '
                    'Problem installing fixture .*')

    def test_handle_integrity_error(self):
        with open(
            p('..', 'smuggler_fixtures', 'garbage', 'invalid_page_dump.json'),
            mode='rb'
        ) as f:
            response = self.client.post(self.url, {
                'uploads': f
            }, follow=True)
        response_messages = list(response.context['messages'])
        self.assertEqual(1, len(response_messages))
        self.assertEqual(messages.ERROR, response_messages[0].level)
        self.assertRegex(response_messages[0].message,
                    r'(?i)An exception occurred while loading data:.*unique.*')

    @override_settings(SMUGGLER_FIXTURE_DIR=p('..', 'smuggler_fixtures'))
    def test_load_from_disk(self):
        self.assertEqual(0, Page.objects.count())
        self.client.post(self.url, {
            'picked_files': p('..', 'smuggler_fixtures', 'page_dump.json')
        }, follow=True)
        self.assertEqual(1, Page.objects.count())

    @override_settings(SMUGGLER_FIXTURE_DIR=p('..', 'smuggler_fixtures'))
    def test_load_from_disk_and_upload(self):
        with open(
            p('..', 'smuggler_fixtures', 'page_dump.json'), mode='rb'
        ) as f:
            response = self.client.post(self.url, {
                'uploads': f,
                'picked_files': p('..', 'smuggler_fixtures', 'page_dump.json')
            }, follow=True)
        response_messages = list(response.context['messages'])
        self.assertEqual(1, len(response_messages))
        self.assertEqual(messages.INFO, response_messages[0].level)
        self.assertEqual(response_messages[0].message,
                         'Successfully imported 2 files. Loaded 2 objects.')

    @override_settings(SMUGGLER_FIXTURE_DIR=p('..', 'smuggler_fixtures'))
    def test_load_and_save(self):
        f = SimpleUploadedFile('uploaded.json',
                               b'[{"pk": 1, "model": "test_app.page",'
                               b' "fields": {"title": "test",'
                               b' "path": "", "body": "test body"}}]')
        self.client.post(self.url, {
            'store': True,
            'uploads': f
        }, follow=True)
        self.assertTrue(os.path.exists(
            p('..', 'smuggler_fixtures', 'uploaded.json')))
        os.unlink(p('..', 'smuggler_fixtures', 'uploaded.json'))



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

from django.urls import include, path

from smuggler.views import dump_app_data, dump_data, dump_model_data, load_data

urlpatterns = [
    path('dump/', dump_data, name='dump-data'),
    path('<slug:app_label>/', include([
        path('dump/', dump_app_data, name='dump-app-data'),
        path(
            '<slug:model_label>/dump/',
            dump_model_data,
            name='dump-model-data'
        ),
    ])),
    path('load/', load_data, name='load-data')
]



import os.path


def p(*args):
    return os.path.abspath(
        os.path.join(os.path.dirname(__file__), *args))


DEBUG = True
STATIC_ROOT = p('')
STATIC_URL = '/static/'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'smuggler.db'
    }
}

SECRET_KEY = 'mAtTzVPOV9JY4eJQfqgW8eAS9DWKnt3MkvvpQI2MzkhAz7z3'

ROOT_URLCONF = 'tests.test_app.urls'

MIDDLEWARE = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

SITE_ID = 1

LOGIN_URL = '/admin/login/'

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.sites',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'smuggler',
    'tests.test_app',
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': (
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.request'
            ),
        },
    },
]

# SMUGGLER_FIXTURE_DIR = p('test_app', 'smuggler_fixtures')



import os.path

from django.contrib.sites.models import Site
from django.db import IntegrityError
from django.test import TestCase, TransactionTestCase

from smuggler.utils import load_fixtures
from tests.test_app.models import Page


def p(*args):
    return os.path.abspath(
        os.path.join(os.path.dirname(__file__), *args))


class SimpleLoadTestCase(TestCase):
    def test_load_page(self):
        self.assertEqual(0, Page.objects.count())
        count = load_fixtures([
            p('..', 'smuggler_fixtures', 'page_dump.json')])
        self.assertEqual(count, 1)
        self.assertEqual('test', Page.objects.get(pk=1).title)

    def test_load_page_and_site(self):
        self.assertEqual(0, Page.objects.count())
        self.assertEqual(Site.objects.get(pk=1).name, 'example.com')
        count = load_fixtures([
            p('..', 'smuggler_fixtures', 'page_dump.json'),
            p('..', 'smuggler_fixtures', 'site_dump.json')
        ])
        self.assertEqual(count, 2)
        self.assertEqual('test', Page.objects.get(pk=1).title)
        self.assertEqual('test.com', Site.objects.get(pk=1).name)

    def test_load_all(self):
        self.assertEqual(0, Page.objects.count())
        self.assertEqual(Site.objects.get(pk=1).name, 'example.com')
        count = load_fixtures([
            p('..', 'smuggler_fixtures', 'all_dump.json')
        ])
        self.assertEqual(count, 2)
        self.assertEqual('test', Page.objects.get(pk=1).title)
        self.assertEqual('test.com', Site.objects.get(pk=1).name)


class TestInvalidLoad(TransactionTestCase):
    def test_load_invalid_data(self):
        self.assertRaises(IntegrityError, load_fixtures,
                          [p('..', 'smuggler_fixtures',
                             'garbage', 'invalid_page_dump.json')])
        self.assertEqual(0, Page.objects.count())



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.
import os.path

from django import forms
from django.conf import settings as django_settings
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.core.serializers import get_serializer_formats
from django.utils.translation import gettext_lazy as _

from smuggler import settings


class MultiFileInput(forms.FileInput):
    def render(self, name, value, attrs=None, **kwargs):
        attrs = attrs or {}
        attrs['multiple'] = 'multiple'
        return super().render(name, None, attrs=attrs, **kwargs)

    def value_from_datadict(self, data, files, name):
        if hasattr(files, 'getlist'):
            return files.getlist(name)
        if name in files:
            return [files.get(name)]
        return []


class MultiFixtureField(forms.FileField):
    widget = MultiFileInput

    def to_python(self, data):
        files = []
        for item in data:
            files.append(super().to_python(item))
        return files

    def validate(self, data):
        super(MultiFixtureField, self).validate(data)
        for upload in data:
            file_format = os.path.splitext(upload.name)[1][1:].lower()
            if file_format not in get_serializer_formats():
                raise forms.ValidationError(
                    _('Invalid file extension: .%(extension)s.') % {
                        'extension': file_format
                    })
        return data


class FixturePathField(forms.MultipleChoiceField, forms.FilePathField):
    widget = FilteredSelectMultiple(_('files'), False)

    def __init__(self, path, match=None, **kwargs):
        match = match or (
            r'(?i)^.+(%s)$' % '|'.join(
                [r'\.%s' % ext for ext in get_serializer_formats()])
        )  # Generate a regex string like: (?i)^.+(\.xml|\.json)$
        super().__init__(path, match=match, **kwargs)
        if not self.required:
            del self.choices[0]  # Remove the empty option


class ImportForm(forms.Form):
    uploads = MultiFixtureField(
        label=_('Upload'),
        required=False
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if settings.SMUGGLER_FIXTURE_DIR:
            self.fields['store'] = forms.BooleanField(
                label=_('Save in fixture directory'),
                required=False,
                help_text=(
                    _('Uploads will be saved to "%(fixture_dir)s".') % {
                        'fixture_dir': settings.SMUGGLER_FIXTURE_DIR
                    })
            )
            self.fields['picked_files'] = FixturePathField(
                settings.SMUGGLER_FIXTURE_DIR,
                label=_('From fixture directory'),
                required=False,
                help_text=(
                    _('Data files from "%(fixture_dir)s".') % {
                        'fixture_dir': settings.SMUGGLER_FIXTURE_DIR
                    })
            )
        else:
            self.fields['uploads'].required = True

    def clean(self):
        super().clean()
        if settings.SMUGGLER_FIXTURE_DIR:
            uploads = self.cleaned_data['uploads']
            picked_files = self.cleaned_data['picked_files']
            if not uploads and not picked_files:
                raise forms.ValidationError(
                    _('At least one fixture file needs to be'
                      ' uploaded or selected.'))
        return self.cleaned_data

    class Media:
        css = {
            'all': ['admin/css/forms.css']
        }
        js = [
            'admin/js/vendor/jquery/jquery%s.js' % (
                '' if django_settings.DEBUG else '.min'
            ),
            'admin/js/jquery.init.js',
            'admin/js/core.js',
            'admin/js/SelectBox.js',
            'admin/js/SelectFilter2.js'
        ]



import json

from io import StringIO

from django.core.management import CommandError
from django.test import TestCase

from smuggler import utils
from tests.test_app.models import Page


class BasicDumpTestCase(TestCase):
    SITE_DUMP = [{
        "model": "sites.site",
        "fields": {
            "domain": "example.com",
            "name": "example.com"
        }
    }]
    PAGE_DUMP = [{
        "pk": 1,
        "model": "test_app.page",
        "fields": {
            "title": "test",
            "path": "",
            "body": "test body",
        }
    }]
    BASIC_DUMP = SITE_DUMP + PAGE_DUMP

    def setUp(self):
        p = Page(title='test', body='test body')
        p.save()

    def normalize(self, out):
        return json.loads(out)

    def test_serialize_exclude(self):
        stream = StringIO()
        utils.serialize_to_response(exclude=['sites', 'auth', 'contenttypes'],
                                    response=stream)
        out = self.normalize(stream.getvalue())
        self.assertEqual(out, self.PAGE_DUMP)

    def test_serialize_include(self):
        stream = StringIO()
        utils.serialize_to_response(app_labels=['sites'], response=stream)
        out = self.normalize(stream.getvalue())
        self.assertEqual(out, self.SITE_DUMP)

    def test_serialize_unknown_app_fail(self):
        self.assertRaises(CommandError, utils.serialize_to_response,
                          ['flatpages'])



from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse


class TestSmugglerViewsRequireAuthentication(TestCase):
    def test_dump_data(self):
        url = reverse('dump-data')
        response = self.client.get(url)
        self.assertRedirects(
            response, '/admin/login/?next=/admin/dump/')

    def test_dump_app_data(self):
        url = reverse('dump-app-data', kwargs={'app_label': 'sites'})
        response = self.client.get(url)
        self.assertRedirects(
            response, '/admin/login/?next=/admin/sites/dump/')

    def test_dump_model_data(self):
        url = reverse('dump-model-data', kwargs={
            'app_label': 'sites',
            'model_label': 'site'
        })
        response = self.client.get(url)
        self.assertRedirects(
            response, '/admin/login/?next=/admin/sites/site/dump/')

    def test_load_data(self):
        url = reverse('load-data')
        response = self.client.get(url, follow=True)
        self.assertRedirects(
            response, '/admin/login/?next=/admin/load/')


class TestSmugglerViewsDeniesNonSuperuser(TestCase):
    def setUp(self):
        staff = User(username='staff')
        staff.set_password('test')
        staff.is_staff = True
        staff.save()
        self.client.login(username='staff', password='test')

    def test_dump_data(self):
        url = reverse('dump-data')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_dump_app_data(self):
        url = reverse('dump-app-data', kwargs={'app_label': 'sites'})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_dump_model_data(self):
        url = reverse('dump-model-data', kwargs={
            'app_label': 'sites',
            'model_label': 'site'
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_load_data(self):
        url = reverse('load-data')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)


class TestSmugglerViewsAllowsSuperuser(TestCase):
    def setUp(self):
        superuser = User(username='superuser')
        superuser.set_password('test')
        superuser.is_staff = True
        superuser.is_superuser = True
        superuser.save()
        self.client.login(username='superuser', password='test')

    def test_dump_data(self):
        url = reverse('dump-data')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_dump_app_data(self):
        url = reverse('dump-app-data', kwargs={'app_label': 'sites'})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_dump_model_data(self):
        url = reverse('dump-model-data', kwargs={
            'app_label': 'sites',
            'model_label': 'site'
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_load_data(self):
        url = reverse('load-data')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)



import os.path

from django.core.files.uploadedfile import SimpleUploadedFile
from django.forms import BooleanField, FilePathField
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.datastructures import MultiValueDict

from smuggler import settings
from smuggler.forms import ImportForm


def p(*args):
    return os.path.abspath(
        os.path.join(os.path.dirname(__file__), *args))


class TestForm(TestCase):
    def test_requires_file(self):
        form = ImportForm({}, {})
        self.assertFalse(form.is_valid())
        self.assertEqual({'uploads': ["This field is required."]},
                         form.errors)

    def test_invalid_file_extension(self):
        f = SimpleUploadedFile('invalid.txt', b'invalid')
        form = ImportForm({}, {
            'uploads': f
        })
        self.assertFalse(form.is_valid())
        self.assertEqual({'uploads': ["Invalid file extension: .txt."]},
                         form.errors)

    def test_valid_file_extension(self):
        f = SimpleUploadedFile('valid.json', b'[]')
        form = ImportForm({}, {
            'uploads': f
        })
        self.assertTrue(form.is_valid())

    def test_valid_uppercase_file_extension(self):
        f = SimpleUploadedFile('valid.JSON', b'[]')
        form = ImportForm({}, {
            'uploads': f
        })
        self.assertTrue(form.is_valid())

    def test_mix_valid_and_invalid(self):
        form = ImportForm({}, MultiValueDict({
            'uploads': [
                SimpleUploadedFile('valid.json', b'[]'),
                SimpleUploadedFile('invalid.txt', b'invalid')
            ]
        }))
        self.assertFalse(form.is_valid())
        self.assertEqual({'uploads': ["Invalid file extension: .txt."]},
                         form.errors)

    @override_settings(SMUGGLER_FIXTURE_DIR=p('..', 'smuggler_fixtures'))
    def test_store_checkbox(self):
        form = ImportForm()
        self.assertIsInstance(form['store'].field, BooleanField)

    @override_settings(SMUGGLER_FIXTURE_DIR=p('..', 'smuggler_fixtures'))
    def test_picked_files(self):
        form = ImportForm()
        self.assertIsInstance(form['picked_files'].field, FilePathField)

    @override_settings(SMUGGLER_FIXTURE_DIR=p('..', 'smuggler_fixtures'))
    def test_requires_at_least_one_field(self):
        form = ImportForm({}, {})
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors, {
            '__all__': [
                'At least one fixture file needs to be uploaded or selected.'
            ]})



#/usr/bin/env python
#
# Copyright (c) 2009-2013 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

import codecs
import os
import sys

from setuptools import find_packages, setup

# Dynamically calculate the version based on smuggler.VERSION.
version = __import__('smuggler').get_version()

if 'publish' in sys.argv:
    os.system('python setup.py sdist bdist_wheel upload')
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()


def read(filepath):
    return codecs.open(filepath, 'r', 'utf-8').read()


setup(
    name='django-smuggler',
    version=version,
    description=('Pluggable application for Django that helps you to '
                 'import/export fixtures via the administration interface'),
    long_description=read(os.path.join(os.path.dirname(__file__), 'README.rst')),
    keywords='django apps tools backup fixtures admin',
    author='Guilherme Gondim',
    author_email='semente+django-smuggler@taurinus.org',
    maintainer='Guilherme Gondim',
    maintainer_email='semente+django-smuggler@taurinus.org',
    license='GNU Lesser General Public License v3 or later (LGPLv3+)',
    url='http://github.com/semente/django-smuggler',
    download_url='http://github.com/semente/django-smuggler/downloads',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    tests_require=[],
)



from django.contrib import admin

from tests.test_app.models import Page


class PageAdmin(admin.ModelAdmin):
    change_list_template = 'smuggler/change_list.html'


admin.site.register(Page, PageAdmin)
